var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { addDays, endOfDay, startOfDay, startOfMonth, endOfMonth, addMonths, startOfWeek, endOfWeek, isSameDay, differenceInCalendarDays, } from 'date-fns';
import { compose } from 'ramda';
import { hasRangeGen } from './types';
export var DEFAULT_WEEK_STARTS_ON = 0;
export var definedsGen = function (_a) {
    var weekStartsOn = _a.weekStartsOn;
    return ({
        startOfWeek: startOfWeek(new Date(), { weekStartsOn: weekStartsOn }),
        endOfWeek: endOfWeek(new Date(), { weekStartsOn: weekStartsOn }),
        startOfLastWeek: startOfWeek(addDays(new Date(), -7), { weekStartsOn: weekStartsOn }),
        endOfLastWeek: endOfWeek(addDays(new Date(), -7), { weekStartsOn: weekStartsOn }),
        startOfToday: startOfDay(new Date()),
        endOfToday: endOfDay(new Date()),
        startOfYesterday: startOfDay(addDays(new Date(), -1)),
        endOfYesterday: endOfDay(addDays(new Date(), -1)),
        startOfMonth: startOfMonth(new Date()),
        endOfMonth: endOfMonth(new Date()),
        startOfLastMonth: startOfMonth(addMonths(new Date(), -1)),
        endOfLastMonth: endOfMonth(addMonths(new Date(), -1)),
    });
};
var defineds = definedsGen({ weekStartsOn: DEFAULT_WEEK_STARTS_ON });
export var getDummyRange = function () {
    return {
        startDate: defineds.startOfToday,
        endDate: null,
    };
};
export function isSameRangeDay(someRange, otherRange) {
    var isSameStart = (someRange.startDate && otherRange.startDate && isSameDay(someRange.startDate, otherRange.startDate))
        || (!someRange.startDate && !otherRange.startDate);
    var isSameEnd = (someRange.endDate && otherRange.endDate && isSameDay(someRange.endDate, otherRange.endDate))
        || (!someRange.endDate && !otherRange.endDate);
    return isSameStart && isSameEnd;
}
export function extractRange(wr) {
    if (hasRangeGen(wr)) {
        return wr.range();
    }
    return wr.range;
}
export var staticRangeHandler = function (withRangeOrRangeGen) { return (__assign(__assign({}, withRangeOrRangeGen), { isSelected: function (range) {
        return isSameRangeDay(range, extractRange(this));
    } })); };
export function createStaticRanges(ranges) {
    return ranges.map(staticRangeHandler);
}
export var defaultStaticRangesGenerator = function (defineds) {
    return createStaticRanges([
        {
            label: 'Last Month',
            range: function () { return ({
                startDate: defineds.startOfLastMonth,
                endDate: defineds.endOfLastMonth,
            }); },
        },
        {
            label: 'Last Week',
            range: function () { return ({
                startDate: defineds.startOfLastWeek,
                endDate: defineds.endOfLastWeek,
            }); },
        },
        {
            label: 'Yesterday',
            range: function () { return ({
                startDate: defineds.startOfYesterday,
                endDate: defineds.endOfYesterday,
            }); },
        },
        {
            label: 'Today',
            range: function () { return ({
                startDate: defineds.startOfToday,
                endDate: defineds.endOfToday,
            }); },
        },
        {
            label: 'This Week',
            range: function () { return ({
                startDate: defineds.startOfWeek,
                endDate: defineds.endOfWeek,
            }); },
        },
        {
            label: 'This Month',
            range: function () { return ({
                startDate: defineds.startOfMonth,
                endDate: defineds.endOfMonth,
            }); },
        },
    ]);
};
export var defaultStaticRangesGen = compose(defaultStaticRangesGenerator, definedsGen);
export var defaultStaticRanges = defaultStaticRangesGenerator(defineds);
export var defaultInputRangesGenerator = function (defineds) { return [
    {
        label: 'days up to today',
        range: function (value) {
            return {
                startDate: addDays(defineds.startOfToday, (Math.max(Number(value), 1) - 1) * -1),
                endDate: defineds.endOfToday,
            };
        },
        getCurrentValue: function (range) {
            if (!range.startDate || !isSameDay(range.startDate, defineds.startOfToday))
                return '-';
            if (!range.startDate)
                return '∞';
            return differenceInCalendarDays(defineds.endOfToday, range.startDate) + 1;
        },
    },
    {
        label: 'days starting today',
        range: function (value) {
            var today = new Date();
            return {
                startDate: today,
                endDate: addDays(today, Math.max(Number(value), 1) - 1),
            };
        },
        getCurrentValue: function (range) {
            if (!range.startDate || !isSameDay(range.startDate, defineds.startOfToday))
                return '-';
            if (!range.endDate)
                return '∞';
            return differenceInCalendarDays(range.endDate, defineds.startOfToday) + 1;
        },
    },
]; };
export var defaultInputRangesGen = compose(defaultInputRangesGenerator, definedsGen);
export var defaultInputRanges = defaultInputRangesGenerator(defineds);
